/*
 *	Header:
 *	File: position.c
 *	Author: Noda Itsuki
 *	Date: 1997/04/1
 *	EndHeader:
 */

#include <stdio.h>
#include <math.h>

#include "libsclient.h"


Bool estimate_current_pos(SeeInfo *sinf, Side side, PosState *pstate)
{
	int i ;
	int out_of_field = FALSE ;
	double dist = 2.0*PITCH_LENGTH ;
	double line_dir, obj_dir ;
	double tmp ;
	SeenObj *line = (SeenObj *)NULL, *flag = (SeenObj *)NULL ;
	SeenObj *goal = (SeenObj *)NULL, *obj ;
	Pos obj_pos ;
	const struct {
		FlagPos	name ;
		Pos		pos ;
	} flags[] = {\
		{FP_L_T, {-PITCH_LENGTH/2.0, -PITCH_WIDTH/2.0}}, \
		{FP_L_B, {-PITCH_LENGTH/2.0, PITCH_WIDTH/2.0}}, \
		{FP_R_T, {PITCH_LENGTH/2.0, -PITCH_WIDTH/2.0}}, \
		{FP_R_B, {PITCH_LENGTH/2.0, PITCH_WIDTH/2.0}}, \
		{FP_C_T, {0.0, -PITCH_WIDTH/2.0}}, \
		{FP_C_B, {0.0, PITCH_WIDTH/2.0}}, \
		{FP_P_L_T, {-PITCH_LENGTH/2.0 + PENALTY_AREA_LENGTH, \
					-PENALTY_AREA_WIDTH/2.0}}, \
		{FP_P_L_C, {-PITCH_LENGTH/2.0 + PENALTY_AREA_LENGTH, 0.0}}, \
		{FP_P_L_B, {-PITCH_LENGTH/2.0 + PENALTY_AREA_LENGTH, \
					PENALTY_AREA_WIDTH/2.0}}, \
		{FP_P_R_T, {PITCH_LENGTH/2.0 - PENALTY_AREA_LENGTH, \
					-PENALTY_AREA_WIDTH/2.0}}, \
		{FP_P_R_C, {PITCH_LENGTH/2.0 - PENALTY_AREA_LENGTH, 0.0}}, \
		{FP_P_R_B, {PITCH_LENGTH/2.0 - PENALTY_AREA_LENGTH, \
					PENALTY_AREA_WIDTH/2.0}}, \
		{FP_Unknown, {0.0, 0.0}} \
	} ;

	for(i = 0; i < sinf->n_obj; i++) {
		if (sinf->obj[i].type == OT_Flag) {
			if (!sinf->obj[i].inViewConeP)
				continue ;
			if (Abs(dist) > Abs(sinf->obj[i].dist)) {
				flag = &(sinf->obj[i]) ;
				dist = sinf->obj[i].dist ;
			}
		}
		else if (sinf->obj[i].type == OT_Line) {
			line = &(sinf->obj[i]) ;
			if (i != sinf->n_obj - 1 && sinf->obj[i+1].type == OT_Line)
				out_of_field = TRUE ;
			break ;
		}
		else if (sinf->obj[i].type == OT_Goal)
			goal = &(sinf->obj[i]) ;
	}

	if (line == (SeenObj *)NULL)
		return FALSE ;

	if (flag != (SeenObj *)NULL) {
		obj = flag ;
		for(i=0; ; i++) {
			if (flags[i].name == FP_Unknown)
				return FALSE ;
			if (flags[i].name == flag->id.fpos) {
				obj_pos.x = flags[i].pos.x ;
				obj_pos.y = flags[i].pos.y ;
				break ;
			}
		}
	}
	else if (goal != (SeenObj *)NULL) {
		obj = goal ;
		if (goal->id.side == S_LEFT)
			obj_pos.x = -PITCH_LENGTH/2.0 ;
		else if (goal->id.side == S_RIGHT)
			obj_pos.x = PITCH_LENGTH/2.0 ;
		else
			return FALSE ;
		obj_pos.y = 0.0 ;
	}
	else
		return FALSE ;

	line_dir = Deg2Rad(line->dir) ;

	switch (line->id.lpos) {
	case LP_T :
		if (out_of_field)  {
			pstate->dir = -line->dir - ((line->dir > 0) ? -180.0 : 0.0) ;
			pstate->pos.y = -PITCH_WIDTH/2.0 - Abs(line->dist * sin(line_dir)) ;
		}
		else {
			pstate->dir = -line->dir - ((line->dir < 0) ? 180.0 : 0.0) ;
			pstate->pos.y = -PITCH_WIDTH/2.0 + Abs(line->dist * sin(line_dir)) ;
		}

		if ((tmp = Pow(obj->dist) - Pow(obj_pos.y - pstate->pos.y)) < 0.0)
			tmp *= -1.0 ;

		obj_dir = obj->dir + pstate->dir ;
		if (obj_dir > 180.0)
			obj_dir -= 360.0 ;
		else if (obj_dir < -180.0)
			obj_dir += 360.0 ;

		if (Abs(obj_dir) < 90.0)
			pstate->pos.x = obj_pos.x - sqrt(tmp) ;
		else
			pstate->pos.x = obj_pos.x + sqrt(tmp) ;
		break ;

	case LP_B :
		if (out_of_field) {
			pstate->dir = -line->dir - ((line->dir < 0) ? 180.0 : 0.0) ;
			pstate->pos.y = PITCH_WIDTH/2.0 + Abs(line->dist * sin(line_dir)) ;
		}
		else {
			pstate->dir = -line->dir - ((line->dir > 0) ? -180.0 : 0.0) ;
			pstate->pos.y = PITCH_WIDTH/2.0 - Abs(line->dist * sin(line_dir)) ;
		}

		if ((tmp = Pow(obj->dist) - Pow(obj_pos.y - pstate->pos.y)) < 0.0)
			tmp *= -1.0 ;

		obj_dir = obj->dir + pstate->dir ;
		if (obj_dir > 180.0)
			obj_dir -= 360.0 ;
		else if (obj_dir < -180.0)
			obj_dir += 360.0 ;

		if (Abs(obj_dir) < 90.0)
			pstate->pos.x = obj_pos.x - sqrt(tmp) ;
		else
			pstate->pos.x = obj_pos.x + sqrt(tmp) ;
		break ;

	case LP_L :
		if (out_of_field) {
			pstate->dir = -line->dir - ((line->dir > 0) ? -90.0 : 90.0) ;
			pstate->pos.x = -PITCH_LENGTH/2.0 - Abs(line->dist * sin(line_dir)) ;
		}
		else {
			pstate->dir = -line->dir - ((line->dir < 0) ? -90.0 : 90.0) ;
			pstate->pos.x = -PITCH_LENGTH/2.0 + Abs(line->dist * sin(line_dir)) ;
		}

		if ((tmp = Pow(obj->dist) - Pow(obj_pos.x - pstate->pos.x)) < 0.0)
			tmp *= -1.0 ;

		obj_dir = obj->dir + pstate->dir ;
		if (obj_dir > 180.0)
			obj_dir -= 360.0 ;
		else if (obj_dir < -180.0)
			obj_dir += 360.0 ;

		if (obj_dir > 0.0)
			pstate->pos.y = obj_pos.y - sqrt(tmp) ;
		else
			pstate->pos.y = obj_pos.y + sqrt(tmp) ;
		break ;

	case LP_R :
		if (out_of_field) {
			pstate->dir = -line->dir - ((line->dir < 0) ? -90.0 : 90.0) ;
			pstate->pos.x = PITCH_LENGTH/2.0 + Abs(line->dist * sin(line_dir)) ;
		}
		else {
			pstate->dir = -line->dir - ((line->dir > 0) ? -90.0 : 90.0) ;
			pstate->pos.x = PITCH_LENGTH/2.0 - Abs(line->dist * sin(line_dir)) ;
		}

		if ((tmp = Pow(obj->dist) - Pow(obj_pos.x - pstate->pos.x)) < 0.0)
			tmp *= -1.0 ;

		obj_dir = obj->dir + pstate->dir ;
		if (obj_dir > 180.0)
			obj_dir -= 360.0 ;
		else if (obj_dir < -180.0)
			obj_dir += 360.0 ;

		if (obj_dir > 0.0)
			pstate->pos.y = obj_pos.y - sqrt(tmp) ;
		else
			pstate->pos.y = obj_pos.y + sqrt(tmp) ;
		break ;

	case LP_Unknown :
		return FALSE ;
	}

	if (side == S_RIGHT) {
		pstate->dir += ((pstate->dir > 0) ? -180.0 : 180.0) ;
		pstate->pos.x *= -1.0 ;
		pstate->pos.y *= -1.0 ;
	}
		
	return TRUE ;
}

